
#include "uniformPerFrameConstants.h"

#define CLOUD_OCTAVES		0.45
#define CLOUD_SHARPNESS		0.00010

highp vec3 PositionTransform(highp vec3 pos, highp float height)
{ highp float dis = length(pos.xz); 
  highp float ang = atan(dis, height); return pos * cos(ang); }

highp float cos_between_vecs(highp vec3 v1, highp vec3 v2)
{ return (v1.x * v2.x + v1.y * v2.y  + v1.z * v2.z) / length(v1) / length(v2); }
highp float randff(highp float n){ return fract(cos(n) * 50.288); }

vec3 jodieReinhardTonemap(vec3 c) {
    float l = dot(c, vec3(0.2126, 0.7152, 0.0722));
    vec3 tc = c / (c + 0.4);
    return mix(c / (l + 1.7), tc, tc);
}

highp float hs(highp float n){return fract(sin(n)*43758.5453123);}

highp float nois(in highp vec2 x)
{highp vec2 p = floor(x); highp vec2 f = fract(x); f = f*f*(3.-2.*f);
highp float n = p.x+p.y*57.;
highp float res = mix(mix(hs(n+0.),hs(n+1.),f.x),mix(hs(n+ 57.),hs(n+ 58.),f.x),f.y);
return res;}

highp float fbm(highp vec2 p)
{ highp float cover = CLOUD_OCTAVES; highp float sharpness = CLOUD_SHARPNESS;
highp float time = TIME; highp float z = 3.8; highp float rz = 0.0; p *= 11.8;
p += vec2(4.0,2.0)*time*0.015; for(int i = 0; i < 5; i++)
{ rz += nois(p)/z; z *= 2.1; p *= 2.5; p -= time*0.018*pow(z,0.6)/2.0; }
highp float den = 4.0 * pow(abs(rz), 2.0);
den *= 2.1; highp float c = den - (1.0 - cover); if( c < 0.0 ) { c = 0.0; }
den = 1.0 - (pow(sharpness, c)); return den; }

vec4 skyColor(highp vec3 p, vec4 fc, vec4 cc){
float rain = smoothstep(.55, .3, FOG_CONTROL.x);
float night = pow(max(min(1.0-fc.r*1.5,1.0),0.0),1.2);
float dusk = pow(max(min(1.0-fc.b*1.2,1.0),0.0),0.5);
float rd = length(p.xz);
vec4 day_color = vec4(0.2,0.5,0.99,1.0)*0.45;
vec4 night_color = vec4(0.1,0.2,0.47,1.0);
vec4 sun_color = cc*FOG_COLOR;
vec4 fc1 = FOG_COLOR;
vec4 rain_color = mix(vec4(2.0),FOG_COLOR,night);

vec4 colors = mix(mix(mix(day_color, sun_color, dusk), night_color, night),rain_color, rain);

//gradient
vec4 grad1 = mix(mix(mix(fc,fc,dusk),fc1,night),fc,rain);
vec4 grad2 = mix(mix(mix(fc,fc,dusk),colors,night),fc,rain);
vec4 grad3 = mix(mix(mix(fc,fc,dusk),colors,night),rain_color,rain);
vec4 grad4 = mix(mix(mix(fc,vec4(1.0,0.6,0.9,1.0),dusk),colors,night),rain_color,rain);
vec4 grad5 = mix(mix(mix(colors,vec4(0.9,0.7,0.8,1.0),dusk),fc1,night),rain_color,rain);
vec4 grad6 = mix(mix(mix(colors,vec4(0.8,0.6,0.7,1.0),dusk),fc1,night),rain_color,rain);
vec4 grad7 = mix(mix(mix(colors,vec4(0.6,0.5,0.8,1.0),dusk),fc1,night),rain_color,rain);
vec4 grad8 = mix(mix(mix(colors,vec4(0.4,0.3,0.7,1.0),dusk),fc1,night),rain_color,rain);
vec4 grad9 = mix(mix(mix(colors,vec4(0.3,0.2,0.6,1.0),dusk),fc1,night),rain_color,rain);
vec4 grad10 = mix(mix(mix(colors,vec4(.0,0.,0.5,1.0),dusk),fc1,night),colors,rain);

vec4 sky_color;
sky_color= mix(colors,grad10,pow(clamp(rd*1.5,0.0,1.0),2.0));
sky_color = mix(sky_color, grad9, pow(clamp(rd*1.47,0.0,1.0),2.2));
sky_color = mix(sky_color, grad8, pow(clamp(rd*1.43,0.0,1.0),2.1));
sky_color = mix(sky_color, grad7, pow(clamp(rd*1.39,0.0,1.0),2.2));
sky_color = mix(sky_color, grad6, pow(clamp(rd*1.35,0.0,1.0),2.1));
sky_color = mix(sky_color, grad5, pow(clamp(rd*1.32,0.0,1.0),2.2));
sky_color = mix(sky_color, grad4, pow(clamp(rd*1.3,0.0,1.0),2.3));
sky_color.rgb = jodieReinhardTonemap(sky_color.rgb);
sky_color = mix(sky_color, grad3, pow(clamp(rd*1.2,0.0,1.0),3.2));
sky_color = mix(sky_color, grad2, pow(clamp(rd*1.1,0.0,1.0),2.0));
sky_color = mix(sky_color, grad1, pow(clamp(rd*1.0,0.0,1.0),2.0));
return sky_color;}


highp vec2 hash22(highp vec2 p)
{
    p = vec2 (
        dot(p, vec2(127.1, 11.7)),
        dot(p, vec2(269.5, 83.3))
    );
    return -1.0 + 2.0 * fract(sin(p) * 43758.5453123);
}

highp float wnoise(highp vec2 p, highp float time)
{ highp vec2 pi = floor(p); highp vec2 pf = p - pi; float minDistance = 5.0;
    for(int i = -1; i <= 1; i++){ for(int j = -1; j <=1; j++)
        {
            highp vec2 offset = vec2(i, j);
            highp vec2 rand = hash22(pi + offset);
           	highp vec2 move = 0.5 + 0.99 * sin(rand * time);
            move = pi + offset + move - p;
            highp float distance = dot(move, move) * 1.0;
            if(distance < minDistance)
            { minDistance = distance; }
        }
    }
    return minDistance;
}

bool getNetherMask(vec4 ambient){
if(ambient.r > ambient.b && ambient.r < .5 && ambient.b < .05){
return true;} else {return false;}}

float rayPos(in vec3 pos, float yPos)
{
vec3 np = normalize(pos);
float sun = abs(1.0-length(np.yz));
vec3 Pos = vec3(smoothstep((9.5-yPos) + abs(np.y)*2.4,0.0,abs(np.z)*1.7));
vec3 flatRange = vec3(max(0.0,sun));
return pow(max(dot(Pos, flatRange), 0.0),5.0);
}

vec3 raySun(in vec3 pos, vec3 x)
{
float specularFlat = rayPos(pos,2.4);
x *= vec3(1.0)*specularFlat;
return x; }

bool uw(vec4 abt){ if(abt.b > abt.r*2.5 &&  abt.b*3.0 > abt.g){ return true; } else { return false; } }

float filmc(float x) {
float A = 1.48; float B = 0.15;
float C = 0.15; float D = 0.65;
float E = 0.05; float F = 0.35;
return ((x * (A * x + C * B) + D * E) / (x * (A * x + B) + D * F)) - E / F;
}

vec3 Hyra(vec3 clr) {
float W = 1.3 / 1.0; float Luma = dot(clr, vec3(0.0, 0.3, 0.3));
vec3 Chroma = clr - Luma; clr = (Chroma * 1.70) + Luma;
clr = vec3(filmc(clr.r), filmc(clr.g), filmc(clr.b)) / filmc(W); return clr;
}

vec3 FOGw(vec3 x)
{ float avg = (x.r + x.g + x.b); x = (((x - avg )*0.87)+avg);//make a greyscale
x = pow(x,vec3(0.3/0.4));//make to foged x.r = (x.r * 1.1)+(x.r+x.g)*(-0.1);
x.g = (x.g * 1.1)+(x.r+x.b)*(-0.1); x.b = (x.b * 1.1)+(x.r+x.g)*(-0.1); return x; }

highp float smoothrand(highp float pos)
{ highp float start = floor(pos);
 highp float smoothy = smoothstep(0.0, 1.0, fract(pos));
 return mix(randff(start), randff(start + 1.0), smoothy); }

highp float akihiro2d(highp float horizont, highp float forward)
{ highp float start = floor(forward);
 highp float smoothy = smoothstep(0.0, 1.0, fract(forward));
 return mix(smoothrand(horizont + randff(start) * 1000.0), smoothrand(horizont + randff(start + 1.0) * 1000.0), smoothy); }

highp float CheapWaterMap(highp vec3 pos)
{ 
highp float time = TIME;
pos.z += time * 0.26;
 highp float wave1 = akihiro2d(pos.z+pos.z+pos.x, pos.x+pos.x+pos.z);
 highp float wave2 = sin(wave1) * 1.0 + 0.4;
float interp = sin(time * 4.2 + pos.x+pos.z+pos.x+pos.z+pos.x+pos.z) * sin(pos.x) * 0.5;
 return mix(wave1, wave2, interp); }

highp vec3 WaterNormalMap(highp vec3 pos)
{ highp float value = (CheapWaterMap(pos + vec3(0.0, 0.0, 0.9)) - CheapWaterMap(pos)) * 1.0; return vec3(0.0, 1.0, -value); }